@@ -23,15 +23,9 @@ extern "C" {
 struct SvtMetadataArray;
 
 // API Version
-#define SVT_AV1_VERSION_MAJOR 0
-#define SVT_AV1_VERSION_MINOR 9
-#define SVT_AV1_VERSION_PATCHLEVEL 0
-
-#ifndef SVT_VERSION_MAJOR
-#define SVT_VERSION_MAJOR SVT_AV1_VERSION_MAJOR
-#define SVT_VERSION_MINOR SVT_AV1_VERSION_MINOR
-#define SVT_VERSION_PATCHLEVEL SVT_AV1_VERSION_PATCHLEVEL
-#endif
+#define SVT_AV1_VERSION_MAJOR 1
+#define SVT_AV1_VERSION_MINOR 2
+#define SVT_AV1_VERSION_PATCHLEVEL 1
 
 #define SVT_AV1_CHECK_VERSION(major, minor, patch)                            \
     (SVT_AV1_VERSION_MAJOR > (major) ||                                       \
@@ -83,14 +77,13 @@ typedef enum EbAv1PictureType {
     EB_AV1_INVALID_PICTURE       = 0xFF
 } EbAv1PictureType;
 
-/** The EbBool type is intended to be used to represent a true or a false
+/** The Bool type is intended to be used to represent a true or a false
 value when passing parameters to and from the eBrisk API.  The
-EbBool is a 32 bit quantity and is aligned on a 32 bit word boundary.
+Bool is a 32 bit quantity and is aligned on a 32 bit word boundary.
 */
-
-#define EbBool uint8_t
-#define EB_FALSE 0
-#define EB_TRUE 1
+typedef uint8_t Bool;
+#define FALSE 0
+#define TRUE 1
 
 typedef struct EbBufferHeaderType {
     // EbBufferHeaderType size
@@ -111,11 +104,11 @@ typedef struct EbBufferHeaderType {
     int64_t  pts;
 
     // pic info
-    uint32_t qp;
-    uint32_t pic_type;
-    uint64_t luma_sse;
-    uint64_t cr_sse;
-    uint64_t cb_sse;
+    uint32_t         qp;
+    EbAv1PictureType pic_type;
+    uint64_t         luma_sse;
+    uint64_t         cr_sse;
+    uint64_t         cb_sse;
     // pic flags
     uint32_t flags;
 
@@ -128,8 +121,8 @@ typedef struct EbBufferHeaderType {
 
 typedef struct EbComponentType {
     uint32_t size;
-    void *   p_component_private;
-    void *   p_application_private;
+    void    *p_component_private;
+    void    *p_application_private;
 } EbComponentType;
 
 typedef enum EbErrorType {
@@ -160,12 +153,6 @@ typedef enum EbAv1SeqProfile {
     PROFESSIONAL_PROFILE = 2
 } EbAv1SeqProfile;
 
-typedef enum AomBitDepth {
-    AOM_BITS_8  = 8, /**<  8 bits */
-    AOM_BITS_10 = 10, /**< 10 bits */
-    AOM_BITS_12 = 12, /**< 12 bits */
-} AomBitDepth;
-
 // For 8-bit and 10-bit packed inputs and outputs, the luma, cb, and cr fields should be used
 //   for the three input picture planes.  However, for 10-bit unpacked planes the
 //   lumaExt, cbExt, and crExt fields should be used hold the extra 2-bits of
@@ -196,11 +183,6 @@ typedef struct EbSvtIOFormat //former EbSvtEncInput
     EbBitDepth    bit_depth;
 } EbSvtIOFormat;
 
-typedef struct BitstreamLevel {
-    uint8_t major;
-    uint8_t minor;
-} BitstreamLevel;
-
 typedef struct EbOperatingParametersInfo {
     /*!<Specifies the time interval between the arrival of the first bit in the
      * smoothing buffer and the subsequent removal of the data that belongs to
@@ -239,11 +221,11 @@ typedef struct EbAV1OperatingPoint {
 
 typedef struct EbColorConfig {
     /*!< bit depth */
-    uint32_t bit_depth;
+    EbBitDepth bit_depth;
 
     /*!< 1: Indicates that the video does not contain U and V color planes.
      *   0: Indicates that the video contains Y, U, and V color planes. */
-    EbBool mono_chrome;
+    Bool mono_chrome;
 
     /*!< Specify the chroma subsampling format */
     uint8_t subsampling_x;
@@ -255,7 +237,7 @@ typedef struct EbColorConfig {
             matrix_coefficients are present. color_description_present_flag
      *   0: Specifies that color_primaries, transfer_characteristics and
             matrix_coefficients are not present */
-    EbBool color_description_present_flag;
+    Bool color_description_present_flag;
 
     /*!< An integer that is defined by the "Color primaries" section of
      * ISO/IEC 23091-4/ITU-T H.273 */
@@ -279,13 +261,13 @@ typedef struct EbColorConfig {
     /*!< 1: Indicates that the U and V planes may have separate delta quantizer
      *   0: Indicates that the U and V planes will share the same delta
             quantizer value */
-    EbBool separate_uv_delta_q;
+    Bool separate_uv_delta_q;
 
 } EbColorConfig;
 
 typedef struct EbTimingInfo {
     /*!< Timing info present flag */
-    EbBool timing_info_present;
+    Bool timing_info_present;
 
     /*!< Number of time units of a clock operating at the frequency time_scale
      * Hz that corresponds to one increment of a clock tick counter*/
@@ -306,28 +288,50 @@ typedef struct EbTimingInfo {
 
 } EbTimingInfo;
 
+// structure to be allocated at the sample application and passed to the library
+// on a per picture basis through the p_app_private field in the EbBufferHeaderType structure
+// this structure and the data inside would be casted, validated, then copied at the
+// svt_av1_enc_send_picture API call
+typedef enum {
+    PRIVATE_DATA, // data to be passed through and written to the bitstream
+    //FILM_GRAIN_PARAM,        // passing film grain parameters per picture
+    REF_FRAME_SCALING_EVENT, // reference frame scaling data per picture
+    PRIVATE_DATA_TYPES // end of private data types
+} PrivDataType;
+typedef struct EbPrivDataNode {
+    PrivDataType           node_type;
+    void                  *data; // pointer to data structure e.g. EbRefFrameScale or AomFilmGrain
+    uint32_t               size; // size of data being sent for the library to know how much to copy
+    struct EbPrivDataNode *next; // pointer to the next node, NULL if done.
+} EbPrivDataNode;
+typedef struct EbRefFrameScale {
+    uint8_t  scale_mode; // scaling mode, support for RESIZE_NONE, RESIZE_FIXED and RESIZE_RANDOM
+    uint32_t scale_denom; // scaling denominator for non-key frame, from 8~16
+    uint32_t scale_kf_denom; // scaling denominator for key frame, from 8~16
+} EbRefFrameScale;
+
 /**
 CPU FLAGS
 */
-typedef uint64_t CPU_FLAGS;
-#define CPU_FLAGS_MMX (1 << 0)
-#define CPU_FLAGS_SSE (1 << 1)
-#define CPU_FLAGS_SSE2 (1 << 2)
-#define CPU_FLAGS_SSE3 (1 << 3)
-#define CPU_FLAGS_SSSE3 (1 << 4)
-#define CPU_FLAGS_SSE4_1 (1 << 5)
-#define CPU_FLAGS_SSE4_2 (1 << 6)
-#define CPU_FLAGS_AVX (1 << 7)
-#define CPU_FLAGS_AVX2 (1 << 8)
-#define CPU_FLAGS_AVX512F (1 << 9)
-#define CPU_FLAGS_AVX512CD (1 << 10)
-#define CPU_FLAGS_AVX512DQ (1 << 11)
-#define CPU_FLAGS_AVX512ER (1 << 12)
-#define CPU_FLAGS_AVX512PF (1 << 13)
-#define CPU_FLAGS_AVX512BW (1 << 14)
-#define CPU_FLAGS_AVX512VL (1 << 15)
-#define CPU_FLAGS_ALL ((CPU_FLAGS_AVX512VL << 1) - 1)
-#define CPU_FLAGS_INVALID (1ULL << (sizeof(CPU_FLAGS) * 8ULL - 1ULL))
+typedef uint64_t EbCpuFlags;
+#define EB_CPU_FLAGS_MMX (1 << 0)
+#define EB_CPU_FLAGS_SSE (1 << 1)
+#define EB_CPU_FLAGS_SSE2 (1 << 2)
+#define EB_CPU_FLAGS_SSE3 (1 << 3)
+#define EB_CPU_FLAGS_SSSE3 (1 << 4)
+#define EB_CPU_FLAGS_SSE4_1 (1 << 5)
+#define EB_CPU_FLAGS_SSE4_2 (1 << 6)
+#define EB_CPU_FLAGS_AVX (1 << 7)
+#define EB_CPU_FLAGS_AVX2 (1 << 8)
+#define EB_CPU_FLAGS_AVX512F (1 << 9)
+#define EB_CPU_FLAGS_AVX512CD (1 << 10)
+#define EB_CPU_FLAGS_AVX512DQ (1 << 11)
+#define EB_CPU_FLAGS_AVX512ER (1 << 12)
+#define EB_CPU_FLAGS_AVX512PF (1 << 13)
+#define EB_CPU_FLAGS_AVX512BW (1 << 14)
+#define EB_CPU_FLAGS_AVX512VL (1 << 15)
+#define EB_CPU_FLAGS_ALL ((EB_CPU_FLAGS_AVX512VL << 1) - 1)
+#define EB_CPU_FLAGS_INVALID (1ULL << (sizeof(EbCpuFlags) * 8ULL - 1ULL))
 
 #ifdef __cplusplus
 }
