@@ -24,7 +24,7 @@
 void svt_av1_reset_loop_restoration(PictureControlSet *piCSetPtr, uint16_t tile_idx);
 
 static void rest_context_dctor(EbPtr p) {
-    EbThreadContext *     thread_context_ptr = (EbThreadContext *)p;
+    EbThreadContext      *thread_context_ptr = (EbThreadContext *)p;
     EntropyCodingContext *obj                = (EntropyCodingContext *)thread_context_ptr->priv;
     EB_FREE_ARRAY(obj);
 }
@@ -32,7 +32,7 @@ static void rest_context_dctor(EbPtr p) {
 /******************************************************
  * Enc Dec Context Constructor
  ******************************************************/
-EbErrorType entropy_coding_context_ctor(EbThreadContext *  thread_context_ptr,
+EbErrorType entropy_coding_context_ctor(EbThreadContext   *thread_context_ptr,
                                         const EbEncHandle *enc_handle_ptr, int index,
                                         int rate_control_index) {
     EntropyCodingContext *context_ptr;
@@ -40,8 +40,9 @@ EbErrorType entropy_coding_context_ctor(EbThreadContext *  thread_context_ptr,
     thread_context_ptr->priv  = context_ptr;
     thread_context_ptr->dctor = rest_context_dctor;
 
-    context_ptr->is_16bit = (EbBool)(
-        enc_handle_ptr->scs_instance_array[0]->scs_ptr->static_config.encoder_bit_depth > EB_8BIT);
+    context_ptr->is_16bit =
+        (Bool)(enc_handle_ptr->scs_instance_array[0]->scs_ptr->static_config.encoder_bit_depth >
+               EB_EIGHT_BIT);
     ;
 
     // Input/Output System Resource Manager FIFOs
@@ -169,7 +170,7 @@ static void reset_entropy_coding_picture(EntropyCodingContext *context_ptr,
     uint16_t tile_idx = 0;
     uint32_t entropy_coding_qp;
 
-    context_ptr->is_16bit = (EbBool)(scs_ptr->static_config.encoder_bit_depth > EB_8BIT);
+    context_ptr->is_16bit = (Bool)(scs_ptr->static_config.encoder_bit_depth > EB_EIGHT_BIT);
     FrameHeader *frm_hdr  = &pcs_ptr->parent_pcs_ptr->frm_hdr;
     // Asuming cb and cr offset to be the same for chroma QP in both slice and pps for lambda computation
     entropy_coding_qp = pcs_ptr->parent_pcs_ptr->frm_hdr.quantization_params.base_q_idx;
@@ -240,24 +241,24 @@ static void reset_entropy_coding_picture(EntropyCodingContext *context_ptr,
 ********************************************************************************/
 void *entropy_coding_kernel(void *input_ptr) {
     // Context & SCS & PCS
-    EbThreadContext *     thread_context_ptr = (EbThreadContext *)input_ptr;
+    EbThreadContext      *thread_context_ptr = (EbThreadContext *)input_ptr;
     EntropyCodingContext *context_ptr        = (EntropyCodingContext *)thread_context_ptr->priv;
 
     // Input
     EbObjectWrapper *rest_results_wrapper_ptr;
 
     // Output
-    EbObjectWrapper *     entropy_coding_results_wrapper_ptr;
+    EbObjectWrapper      *entropy_coding_results_wrapper_ptr;
     EntropyCodingResults *entropy_coding_results_ptr;
 
     for (;;) {
         // Get Mode Decision Results
         EB_GET_FULL_OBJECT(context_ptr->enc_dec_input_fifo_ptr, &rest_results_wrapper_ptr);
 
-        RestResults *      rest_results_ptr = (RestResults *)rest_results_wrapper_ptr->object_ptr;
+        RestResults       *rest_results_ptr = (RestResults *)rest_results_wrapper_ptr->object_ptr;
         PictureControlSet *pcs_ptr          = (PictureControlSet *)
                                          rest_results_ptr->pcs_wrapper_ptr->object_ptr;
-        SequenceControlSet *scs_ptr = (SequenceControlSet *)pcs_ptr->scs_wrapper_ptr->object_ptr;
+        SequenceControlSet *scs_ptr = pcs_ptr->scs_ptr;
         // SB Constants
 
         uint8_t sb_sz = (uint8_t)scs_ptr->sb_size_pix;
@@ -277,15 +278,17 @@ void *entropy_coding_kernel(void *input_ptr) {
             scs_ptr->seq_header.sb_size_log2;
 
         uint16_t tile_width_in_sb = (cm->tiles_info.tile_col_start_mi[tile_col + 1] -
-            cm->tiles_info.tile_col_start_mi[tile_col]) >> scs_ptr->seq_header.sb_size_log2;
+                                     cm->tiles_info.tile_col_start_mi[tile_col]) >>
+            scs_ptr->seq_header.sb_size_log2;
         uint16_t tile_height_in_sb = (cm->tiles_info.tile_row_start_mi[tile_row + 1] -
-            cm->tiles_info.tile_row_start_mi[tile_row]) >> scs_ptr->seq_header.sb_size_log2;
+                                      cm->tiles_info.tile_row_start_mi[tile_row]) >>
+            scs_ptr->seq_header.sb_size_log2;
 
-        EbBool   frame_entropy_done = EB_FALSE;
+        Bool frame_entropy_done = FALSE;
 
         svt_block_on_mutex(pcs_ptr->entropy_coding_pic_mutex);
         if (pcs_ptr->entropy_coding_pic_reset_flag) {
-            pcs_ptr->entropy_coding_pic_reset_flag = EB_FALSE;
+            pcs_ptr->entropy_coding_pic_reset_flag = FALSE;
 
             reset_entropy_coding_picture(context_ptr, pcs_ptr, scs_ptr);
         }
@@ -293,12 +296,14 @@ void *entropy_coding_kernel(void *input_ptr) {
 
 #if TURN_OFF_EC_FIRST_PASS
         if (scs_ptr->static_config.pass != ENC_FIRST_PASS &&
-            !(!pcs_ptr->parent_pcs_ptr->is_used_as_reference_flag && scs_ptr->rc_stat_gen_pass_mode && !pcs_ptr->parent_pcs_ptr->first_frame_in_minigop)) {
+            !is_pic_skipped(pcs_ptr->parent_pcs_ptr)) {
 #endif
             for (uint32_t y_sb_index = 0; y_sb_index < tile_height_in_sb; ++y_sb_index) {
                 for (uint32_t x_sb_index = 0; x_sb_index < tile_width_in_sb; ++x_sb_index) {
-                    uint16_t    sb_index = (uint16_t)((x_sb_index + tile_sb_start_x) + (y_sb_index + tile_sb_start_y) * pic_width_in_sb);
-                    SuperBlock *sb_ptr = pcs_ptr->sb_ptr_array[sb_index];
+                    uint16_t    sb_index = (uint16_t)((x_sb_index + tile_sb_start_x) +
+                                                   (y_sb_index + tile_sb_start_y) *
+                                                       pic_width_in_sb);
+                    SuperBlock *sb_ptr   = pcs_ptr->sb_ptr_array[sb_index];
 
                     context_ptr->sb_origin_x = (x_sb_index + tile_sb_start_x) << sb_size_log2;
                     context_ptr->sb_origin_y = (y_sb_index + tile_sb_start_y) << sb_size_log2;
@@ -307,28 +312,29 @@ void *entropy_coding_kernel(void *input_ptr) {
                         context_ptr->tok = pcs_ptr->tile_tok[tile_row][tile_col];
                     }
 
-                    EbPictureBufferDesc *coeff_picture_ptr = pcs_ptr->parent_pcs_ptr->enc_dec_ptr->quantized_coeff[sb_index];
+                    EbPictureBufferDesc *coeff_picture_ptr =
+                        pcs_ptr->parent_pcs_ptr->enc_dec_ptr->quantized_coeff[sb_index];
                     write_sb(context_ptr,
-                        sb_ptr,
-                        pcs_ptr,
-                        tile_idx,
-                        pcs_ptr->entropy_coding_info[tile_idx]->entropy_coder_ptr,
-                        coeff_picture_ptr);
+                             sb_ptr,
+                             pcs_ptr,
+                             tile_idx,
+                             pcs_ptr->entropy_coding_info[tile_idx]->entropy_coder_ptr,
+                             coeff_picture_ptr);
                 }
             }
 #if TURN_OFF_EC_FIRST_PASS
         }
 #endif
-        EbBool pic_ready = EB_TRUE;
+        Bool pic_ready = TRUE;
 
         // Current tile ready
         encode_slice_finish(pcs_ptr->entropy_coding_info[tile_idx]->entropy_coder_ptr);
 
         svt_block_on_mutex(pcs_ptr->entropy_coding_pic_mutex);
-        pcs_ptr->entropy_coding_info[tile_idx]->entropy_coding_tile_done = EB_TRUE;
+        pcs_ptr->entropy_coding_info[tile_idx]->entropy_coding_tile_done = TRUE;
         for (uint16_t i = 0; i < tile_cnt; i++) {
-            if (pcs_ptr->entropy_coding_info[i]->entropy_coding_tile_done == EB_FALSE) {
-                pic_ready = EB_FALSE;
+            if (pcs_ptr->entropy_coding_info[i]->entropy_coding_tile_done == FALSE) {
+                pic_ready = FALSE;
                 break;
             }
         }
@@ -336,13 +342,15 @@ void *entropy_coding_kernel(void *input_ptr) {
         if (pic_ready) {
             if (pcs_ptr->parent_pcs_ptr->superres_total_recode_loop == 0) {
                 // Release the List 0 Reference Pictures
-                for (uint32_t ref_idx = 0; ref_idx < pcs_ptr->parent_pcs_ptr->ref_list0_count; ++ref_idx) {
+                for (uint32_t ref_idx = 0; ref_idx < pcs_ptr->parent_pcs_ptr->ref_list0_count;
+                     ++ref_idx) {
                     if (pcs_ptr->ref_pic_ptr_array[0][ref_idx] != NULL) {
                         svt_release_object(pcs_ptr->ref_pic_ptr_array[0][ref_idx]);
                     }
                 }
                 // Release the List 1 Reference Pictures
-                for (uint32_t ref_idx = 0; ref_idx < pcs_ptr->parent_pcs_ptr->ref_list1_count; ++ref_idx) {
+                for (uint32_t ref_idx = 0; ref_idx < pcs_ptr->parent_pcs_ptr->ref_list1_count;
+                     ++ref_idx) {
                     if (pcs_ptr->ref_pic_ptr_array[1][ref_idx] != NULL) {
                         svt_release_object(pcs_ptr->ref_pic_ptr_array[1][ref_idx]);
                     }
@@ -352,13 +360,15 @@ void *entropy_coding_kernel(void *input_ptr) {
                 if (pcs_ptr->tile_tok[0][0])
                     EB_FREE_ARRAY(pcs_ptr->tile_tok[0][0]);
             }
-            frame_entropy_done = EB_TRUE;
+            frame_entropy_done = TRUE;
         }
 
         if (frame_entropy_done) {
             // Get Empty Entropy Coding Results
-            svt_get_empty_object(context_ptr->entropy_coding_output_fifo_ptr, &entropy_coding_results_wrapper_ptr);
-            entropy_coding_results_ptr = (EntropyCodingResults *)entropy_coding_results_wrapper_ptr->object_ptr;
+            svt_get_empty_object(context_ptr->entropy_coding_output_fifo_ptr,
+                                 &entropy_coding_results_wrapper_ptr);
+            entropy_coding_results_ptr = (EntropyCodingResults *)
+                                             entropy_coding_results_wrapper_ptr->object_ptr;
             entropy_coding_results_ptr->pcs_wrapper_ptr = rest_results_ptr->pcs_wrapper_ptr;
 
             // Post EntropyCoding Results
