@@ -17,6 +17,8 @@
 #include "EbSequenceControlSet.h"
 #include "EbPictureBufferDesc.h"
 #include "EbUtility.h"
+#include "EbResourceCoordinationProcess.h"
+#include "EbModeDecisionConfigurationProcess.h"
 
 void set_tile_info(PictureParentControlSet *pcs_ptr);
 
@@ -26,10 +28,6 @@ void *svt_aom_malloc(size_t size);
 
 EbErrorType svt_av1_alloc_restoration_buffers(PictureControlSet *pcs, Av1Common *cm);
 EbErrorType svt_av1_hash_table_create(HashTable *p_hash_table);
-uint8_t get_disallow_below_16x16_picture_level(EbEncMode enc_mode, EbInputResolution resolution,
-                                               EB_SLICE slice_type, uint8_t sc_class1,
-                                               uint8_t is_used_as_reference_flag,
-                                               uint8_t temporal_layer_index);
 
 static void set_restoration_unit_size(int32_t width, int32_t height, int32_t sx, int32_t sy,
                                       RestorationInfo *rst) {
@@ -87,7 +85,17 @@ void get_max_allocated_me_refs(uint8_t ref_count_used_list0, uint8_t ref_count_u
         (ref_count_used_list0 * ref_count_used_list1) + (ref_count_used_list0 - 1) +
         (ref_count_used_list1 == 3 ? 1 : 0);
 }
-uint8_t get_enable_me_16x16(EbEncMode enc_mode) {
+// use this function to set the enable_me_8x8 level
+static uint8_t get_enable_me_8x8(EncMode enc_mode) {
+    uint8_t enable_me_8x8 = 0;
+    if (enc_mode <= ENC_M11)
+        enable_me_8x8 = 1;
+    else
+        enable_me_8x8 = 0;
+
+    return enable_me_8x8;
+}
+uint8_t get_enable_me_16x16(EncMode enc_mode) {
     uint8_t enable_me_16x16;
 
     if (enc_mode <= ENC_M12)
@@ -110,13 +118,8 @@ EbErrorType me_sb_results_ctor(MeSbResults *obj_ptr, PictureControlSetInitData *
     derive_input_resolution(&resolution,
                             init_data_ptr->picture_width * init_data_ptr->picture_height);
     uint8_t number_of_pus = get_enable_me_16x16(init_data_ptr->enc_mode)
-        ?
-        !get_disallow_below_16x16_picture_level(
-            init_data_ptr->enc_mode, resolution, B_SLICE, 0, 1, 0)
-            ? SQUARE_PU_COUNT
-            : MAX_SB64_PU_COUNT_NO_8X8
-        :
-        MAX_SB64_PU_COUNT_WO_16X16;
+        ? get_enable_me_8x8(init_data_ptr->enc_mode) ? SQUARE_PU_COUNT : MAX_SB64_PU_COUNT_NO_8X8
+        : MAX_SB64_PU_COUNT_WO_16X16;
 
     EB_MALLOC_ARRAY(obj_ptr->me_mv_array, number_of_pus * max_ref_to_alloc);
     EB_MALLOC_ARRAY(obj_ptr->me_candidate_array, number_of_pus * max_cand_to_alloc);
@@ -209,12 +212,13 @@ void picture_control_set_dctor(EbPtr p) {
     EB_FREE_ARRAY(obj->sb_intra);
     EB_FREE_ARRAY(obj->sb_skip);
     EB_FREE_ARRAY(obj->sb_64x64_mvp);
+    EB_FREE_ARRAY(obj->sb_count_nz_coeffs);
     EB_DELETE(obj->bitstream_ptr);
     EB_DELETE_PTR_ARRAY(obj->entropy_coding_info, tile_cnt);
 
     const int32_t num_planes = 3; // av1_num_planes(cm);
     for (int32_t pl = 0; pl < num_planes; ++pl) {
-        RestorationInfo *            ri         = obj->rst_info + pl;
+        RestorationInfo             *ri         = obj->rst_info + pl;
         RestorationStripeBoundaries *boundaries = &ri->boundaries;
         EB_FREE_ARRAY(ri->unit_info);
         EB_FREE(boundaries->stripe_boundary_above);
@@ -279,7 +283,7 @@ EbErrorType create_neighbor_array_units(InitData *data, size_t count) {
 }
 
 EbErrorType rtime_alloc_palette_tokens(SequenceControlSet *scs_ptr,
-                                       PictureControlSet * child_pcs_ptr) {
+                                       PictureControlSet  *child_pcs_ptr) {
     if (child_pcs_ptr->parent_pcs_ptr->frm_hdr.allow_screen_content_tools) {
         if (scs_ptr->static_config.screen_content_mode) {
             uint32_t     mi_cols = scs_ptr->max_input_luma_width >> MI_SIZE_LOG2;
@@ -303,12 +307,14 @@ EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data_ptr) {
     // Max/Min CU Sizes
 
     // SBs
-    const uint16_t picture_sb_width = (uint16_t)(
-        (init_data_ptr->picture_width + init_data_ptr->sb_sz - 1) / init_data_ptr->sb_sz);
-    const uint16_t picture_sb_height = (uint16_t)(
-        (init_data_ptr->picture_height + init_data_ptr->sb_sz - 1) / init_data_ptr->sb_sz);
-    uint16_t sb_index;
-    EbBool is_16bit = init_data_ptr->bit_depth > 8 ? EB_TRUE : EB_FALSE;
+    const uint16_t picture_sb_width  = (uint16_t)((init_data_ptr->picture_width +
+                                                  init_data_ptr->sb_sz - 1) /
+                                                 init_data_ptr->sb_sz);
+    const uint16_t picture_sb_height = (uint16_t)((init_data_ptr->picture_height +
+                                                   init_data_ptr->sb_sz - 1) /
+                                                  init_data_ptr->sb_sz);
+    uint16_t       sb_index;
+    Bool           is_16bit = init_data_ptr->bit_depth > 8 ? TRUE : FALSE;
 
     //object_ptr->tile_row_count  = init_data_ptr->tile_row_count;
     //object_ptr->tile_column_count = init_data_ptr->tile_column_count;
@@ -322,13 +328,14 @@ EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data_ptr) {
     input_pic_buf_desc_init_data.buffer_enable_mask = PICTURE_BUFFER_DESC_FULL_MASK;
     input_pic_buf_desc_init_data.color_format       = init_data_ptr->color_format;
     uint16_t padding                                = init_data_ptr->sb_size_pix + 32;
-    input_pic_buf_desc_init_data.left_padding       = padding;
-    input_pic_buf_desc_init_data.right_padding      = padding;
-    input_pic_buf_desc_init_data.top_padding        = padding;
-    input_pic_buf_desc_init_data.bot_padding        = padding;
-    input_pic_buf_desc_init_data.split_mode         = EB_FALSE;
-
-    //object_ptr->scs_wrapper_ptr = (EbObjectWrapper *)NULL;
+    if (init_data_ptr->is_scale) {
+        padding += init_data_ptr->sb_size_pix;
+    }
+    input_pic_buf_desc_init_data.left_padding  = padding;
+    input_pic_buf_desc_init_data.right_padding = padding;
+    input_pic_buf_desc_init_data.top_padding   = padding;
+    input_pic_buf_desc_init_data.bot_padding   = padding;
+    input_pic_buf_desc_init_data.split_mode    = FALSE;
 
     object_ptr->recon_picture16bit_ptr = (EbPictureBufferDesc *)NULL;
     object_ptr->recon_picture_ptr      = (EbPictureBufferDesc *)NULL; //OMK
@@ -348,7 +355,7 @@ EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data_ptr) {
                svt_recon_picture_buffer_desc_ctor,
                (EbPtr)&input_pic_buf_desc_init_data);
         if (init_data_ptr->is_16bit_pipeline) {
-            input_pic_buf_desc_init_data.bit_depth = EB_16BIT;
+            input_pic_buf_desc_init_data.bit_depth = EB_SIXTEEN_BIT;
             EB_NEW(object_ptr->recon_picture16bit_ptr,
                    svt_recon_picture_buffer_desc_ctor,
                    (EbPtr)&input_pic_buf_desc_init_data);
@@ -366,13 +373,13 @@ EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data_ptr) {
     coeff_init_data.buffer_enable_mask = PICTURE_BUFFER_DESC_FULL_MASK;
     coeff_init_data.max_width          = init_data_ptr->sb_size_pix;
     coeff_init_data.max_height         = init_data_ptr->sb_size_pix;
-    coeff_init_data.bit_depth          = EB_32BIT;
+    coeff_init_data.bit_depth          = EB_THIRTYTWO_BIT;
     coeff_init_data.color_format       = init_data_ptr->color_format;
     coeff_init_data.left_padding       = 0;
     coeff_init_data.right_padding      = 0;
     coeff_init_data.top_padding        = 0;
     coeff_init_data.bot_padding        = 0;
-    coeff_init_data.split_mode         = EB_FALSE;
+    coeff_init_data.split_mode         = FALSE;
     for (sb_index = 0; sb_index < object_ptr->sb_total_count_unscaled; ++sb_index) {
         EB_NEW(object_ptr->quantized_coeff[sb_index], //OMK2
                svt_picture_buffer_desc_ctor,
@@ -382,12 +389,8 @@ EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data_ptr) {
     return EB_ErrorNone;
 }
 
-uint8_t get_dlf_level(EbEncMode enc_mode, uint8_t is_used_as_reference_flag);
-
-uint8_t get_enable_restoration(EbEncMode enc_mode, int8_t config_enable_restoration,
-                               uint8_t input_resolution);
-uint8_t get_disallow_4x4(EbEncMode enc_mode, EB_SLICE slice_type);
-uint8_t get_disallow_nsq(EbEncMode enc_mode);
+uint8_t get_disallow_4x4(EncMode enc_mode, SliceType slice_type);
+uint8_t get_disallow_nsq(EncMode enc_mode);
 
 uint32_t get_out_buffer_size(uint32_t picture_width, uint32_t picture_height) {
     uint32_t frame_size = picture_width * picture_height * 3 / 2; //assuming 4:2:0;
@@ -405,35 +408,39 @@ EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object
     // Max/Min CU Sizes
     const uint32_t max_blk_size = init_data_ptr->sb_size_pix;
     // SBs
-    const uint16_t picture_sb_width = (uint16_t)(
-        (init_data_ptr->picture_width + init_data_ptr->sb_sz - 1) / init_data_ptr->sb_sz);
-    const uint16_t picture_sb_height = (uint16_t)(
-        (init_data_ptr->picture_height + init_data_ptr->sb_sz - 1) / init_data_ptr->sb_sz);
-    uint16_t    sb_index;
-    uint16_t    sb_origin_x;
-    uint16_t    sb_origin_y;
-    EbErrorType return_error;
+    const uint16_t picture_sb_width  = (uint16_t)((init_data_ptr->picture_width +
+                                                  init_data_ptr->sb_sz - 1) /
+                                                 init_data_ptr->sb_sz);
+    const uint16_t picture_sb_height = (uint16_t)((init_data_ptr->picture_height +
+                                                   init_data_ptr->sb_sz - 1) /
+                                                  init_data_ptr->sb_sz);
+    uint16_t       sb_index;
+    uint16_t       sb_origin_x;
+    uint16_t       sb_origin_y;
+    EbErrorType    return_error;
 
-    EbBool         is_16bit      = init_data_ptr->bit_depth > 8 ? EB_TRUE : EB_FALSE;
+    Bool           is_16bit      = init_data_ptr->bit_depth > 8 ? TRUE : FALSE;
     const uint16_t subsampling_x = (init_data_ptr->color_format == EB_YUV444 ? 1 : 2) - 1;
     const uint16_t subsampling_y = (init_data_ptr->color_format >= EB_YUV422 ? 1 : 2) - 1;
 
     uint32_t total_tile_cnt = init_data_ptr->tile_row_count * init_data_ptr->tile_column_count;
     uint32_t tile_idx       = 0;
 
-    uint32_t output_buffer_size = get_out_buffer_size(init_data_ptr->picture_width,
+    uint32_t output_buffer_size   = get_out_buffer_size(init_data_ptr->picture_width,
                                                       init_data_ptr->picture_height);
     object_ptr->tile_row_count    = init_data_ptr->tile_row_count;
     object_ptr->tile_column_count = init_data_ptr->tile_column_count;
 
     object_ptr->dctor = picture_control_set_dctor;
 
+    object_ptr->hash_table.p_lookup_table = NULL;
+
     // Init Picture Init data
     uint16_t padding = init_data_ptr->sb_size_pix + 32;
 
     coeff_buffer_desc_init_data.max_width          = init_data_ptr->picture_width;
     coeff_buffer_desc_init_data.max_height         = init_data_ptr->picture_height;
-    coeff_buffer_desc_init_data.bit_depth          = EB_16BIT;
+    coeff_buffer_desc_init_data.bit_depth          = EB_SIXTEEN_BIT;
     coeff_buffer_desc_init_data.buffer_enable_mask = PICTURE_BUFFER_DESC_FULL_MASK;
     coeff_buffer_desc_init_data.color_format       = init_data_ptr->color_format;
 
@@ -441,34 +448,16 @@ EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object
     coeff_buffer_desc_init_data.right_padding     = padding;
     coeff_buffer_desc_init_data.top_padding       = padding;
     coeff_buffer_desc_init_data.bot_padding       = padding;
-    coeff_buffer_desc_init_data.split_mode        = EB_FALSE;
+    coeff_buffer_desc_init_data.split_mode        = FALSE;
     coeff_buffer_desc_init_data.is_16bit_pipeline = init_data_ptr->is_16bit_pipeline;
-
-    object_ptr->scs_wrapper_ptr = (EbObjectWrapper *)NULL;
-
-    object_ptr->color_format = init_data_ptr->color_format;
-    uint8_t lf_recon_needed  = 0;
-    // recon output will also use temp_lf_recon_picture_ptr for adding film grain noise
-    if (init_data_ptr->tile_row_count > 0 || init_data_ptr->tile_column_count > 0 ||
-        (init_data_ptr->static_config.film_grain_denoise_strength > 0 &&
-         init_data_ptr->static_config.recon_enabled))
-        lf_recon_needed = 1;
-    else
-        for (uint8_t is_used_as_reference_flag = 0; is_used_as_reference_flag < 2;
-             is_used_as_reference_flag++) {
-            lf_recon_needed = get_dlf_level(init_data_ptr->enc_mode, is_used_as_reference_flag) == 1
-                ? 1
-                : 0;
-            if (lf_recon_needed)
-                break;
-        }
-    object_ptr->temp_lf_recon_picture16bit_ptr = (EbPictureBufferDesc *)NULL;
-    object_ptr->temp_lf_recon_picture_ptr      = (EbPictureBufferDesc *)NULL;
-    if (lf_recon_needed) {
-    }
+    object_ptr->color_format                      = init_data_ptr->color_format;
+    object_ptr->temp_lf_recon_picture16bit_ptr    = (EbPictureBufferDesc *)NULL;
+    object_ptr->temp_lf_recon_picture_ptr         = (EbPictureBufferDesc *)NULL;
+    object_ptr->scaled_input_picture_ptr          = (EbPictureBufferDesc *)NULL;
     if (get_enable_restoration(init_data_ptr->enc_mode,
                                init_data_ptr->static_config.enable_restoration_filtering,
-                               init_data_ptr->input_resolution)) {
+                               init_data_ptr->input_resolution,
+                               init_data_ptr->static_config.fast_decode)) {
         set_restoration_unit_size(init_data_ptr->picture_width,
                                   init_data_ptr->picture_height,
                                   1,
@@ -517,16 +506,18 @@ EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object
     sb_origin_x = 0;
     sb_origin_y = 0;
 
-    const uint16_t picture_sb_w = (uint16_t)(
-        (init_data_ptr->picture_width + init_data_ptr->sb_size_pix - 1) /
-        init_data_ptr->sb_size_pix);
-    const uint16_t picture_sb_h = (uint16_t)(
-        (init_data_ptr->picture_height + init_data_ptr->sb_size_pix - 1) /
-        init_data_ptr->sb_size_pix);
-    const uint16_t all_sb = picture_sb_w * picture_sb_h;
+    const uint16_t picture_sb_w = (uint16_t)((init_data_ptr->picture_width +
+                                              init_data_ptr->sb_size_pix - 1) /
+                                             init_data_ptr->sb_size_pix);
+    const uint16_t picture_sb_h = (uint16_t)((init_data_ptr->picture_height +
+                                              init_data_ptr->sb_size_pix - 1) /
+                                             init_data_ptr->sb_size_pix);
+    const uint16_t all_sb       = picture_sb_w * picture_sb_h;
 
     object_ptr->sb_total_count_pix = all_sb;
 
+    EB_MALLOC_ARRAY(object_ptr->sb_count_nz_coeffs, object_ptr->sb_total_count_pix);
+
     for (sb_index = 0; sb_index < all_sb; ++sb_index) {
         EB_NEW(object_ptr->sb_ptr_array[sb_index],
                largest_coding_unit_ctor,
@@ -695,8 +686,7 @@ EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object
                     PU_NEIGHBOR_ARRAY_GRANULARITY,
                     PU_NEIGHBOR_ARRAY_GRANULARITY,
                     NEIGHBOR_ARRAY_UNIT_TOP_AND_LEFT_ONLY_MASK,
-                }
-            };
+                }};
             return_error = create_neighbor_array_units(data0, DIM(data0));
             if (return_error == EB_ErrorInsufficientResources)
                 return EB_ErrorInsufficientResources;
@@ -1193,7 +1183,7 @@ EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object
 
     // If NSQ is allowed, then need a 4x4 MI grid because 8x8 NSQ shapes will require 4x4 granularity
     uint8_t disallow_4x4 = get_disallow_nsq(init_data_ptr->enc_mode);
-    for (EB_SLICE slice_type = 0; slice_type < IDR_SLICE + 1; slice_type++)
+    for (SliceType slice_type = 0; slice_type < IDR_SLICE + 1; slice_type++)
         disallow_4x4 = MIN(disallow_4x4, get_disallow_4x4(init_data_ptr->enc_mode, slice_type));
 
     object_ptr->disallow_4x4_all_frames = disallow_4x4;
@@ -1230,9 +1220,11 @@ EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object
 
         EB_CALLOC_ALIGNED_ARRAY(object_ptr->tpl_mvs, mem_size);
     }
+
     if (get_enable_restoration(init_data_ptr->enc_mode,
                                init_data_ptr->static_config.enable_restoration_filtering,
-                               init_data_ptr->input_resolution))
+                               init_data_ptr->input_resolution,
+                               init_data_ptr->static_config.fast_decode))
         EB_MALLOC_ALIGNED(object_ptr->rst_tmpbuf, RESTORATION_TMPBUF_SIZE);
 
     return EB_ErrorNone;
@@ -1263,7 +1255,9 @@ static void picture_parent_control_set_dctor(EbPtr ptr) {
     if (obj->is_chroma_downsampled_picture_ptr_owner)
         EB_DELETE(obj->chroma_downsampled_picture_ptr);
 
-    EB_FREE_2D(obj->variance);
+    if (obj->variance)
+        EB_FREE_2D(obj->variance);
+
     if (obj->picture_histogram) {
         for (int region_in_picture_width_index = 0;
              region_in_picture_width_index < MAX_NUMBER_OF_REGIONS_IN_WIDTH;
@@ -1272,8 +1266,8 @@ static void picture_parent_control_set_dctor(EbPtr ptr) {
                 for (int region_in_picture_height_index = 0;
                      region_in_picture_height_index < MAX_NUMBER_OF_REGIONS_IN_HEIGHT;
                      region_in_picture_height_index++) {
-                    EB_FREE_2D(obj->picture_histogram[region_in_picture_width_index]
-                                                     [region_in_picture_height_index]);
+                    EB_FREE_ARRAY(obj->picture_histogram[region_in_picture_width_index]
+                                                        [region_in_picture_height_index]);
                 }
             }
             EB_FREE_PTR_ARRAY(obj->picture_histogram[region_in_picture_width_index],
@@ -1297,9 +1291,6 @@ static void picture_parent_control_set_dctor(EbPtr ptr) {
     EB_FREE_ARRAY(obj->me_8x8_distortion);
 
     EB_FREE_ARRAY(obj->me_8x8_cost_variance);
-    // SB noise variance array
-    EB_FREE_ARRAY(obj->sb_depth_mode_array);
-
     if (obj->av1_cm) {
         EB_FREE_ARRAY(obj->av1_cm->frame_to_show);
         if (obj->av1_cm->rst_frame.buffer_alloc_sz) {
@@ -1319,7 +1310,7 @@ static void picture_parent_control_set_dctor(EbPtr ptr) {
     EB_DESTROY_MUTEX(obj->first_pass_mutex);
     if (obj->is_pcs_sb_params)
         svt_pcs_sb_structs_dctor(obj);
-    if (obj->frame_superres_enabled) {
+    if (obj->frame_superres_enabled || obj->frame_resize_enabled) {
         EB_DELETE(obj->enhanced_downscaled_picture_ptr);
     }
     EB_DESTROY_SEMAPHORE(obj->tpl_disp_done_semaphore);
@@ -1332,16 +1323,17 @@ EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr,
                                             EbPtr                    object_init_data_ptr) {
     PictureControlSetInitData *init_data_ptr    = (PictureControlSetInitData *)object_init_data_ptr;
     EbErrorType                return_error     = EB_ErrorNone;
-    const uint16_t             picture_sb_width = (uint16_t)(
-        (init_data_ptr->picture_width + init_data_ptr->sb_sz - 1) / init_data_ptr->sb_sz);
-    const uint16_t picture_sb_height = (uint16_t)(
-        (init_data_ptr->picture_height + init_data_ptr->sb_sz - 1) / init_data_ptr->sb_sz);
+    const uint16_t             picture_sb_width = (uint16_t)((init_data_ptr->picture_width +
+                                                  init_data_ptr->sb_sz - 1) /
+                                                 init_data_ptr->sb_sz);
+    const uint16_t             picture_sb_height = (uint16_t)((init_data_ptr->picture_height +
+                                                   init_data_ptr->sb_sz - 1) /
+                                                  init_data_ptr->sb_sz);
     const uint16_t subsampling_x = (init_data_ptr->color_format == EB_YUV444 ? 1 : 2) - 1;
     const uint16_t subsampling_y = (init_data_ptr->color_format >= EB_YUV422 ? 1 : 2) - 1;
 
     object_ptr->dctor = picture_parent_control_set_dctor;
 
-    object_ptr->scs_wrapper_ptr                 = (EbObjectWrapper *)NULL;
     object_ptr->input_picture_wrapper_ptr       = (EbObjectWrapper *)NULL;
     object_ptr->reference_picture_wrapper_ptr   = (EbObjectWrapper *)NULL;
     object_ptr->enhanced_picture_ptr            = (EbPictureBufferDesc *)NULL;
@@ -1359,11 +1351,11 @@ EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr,
         input_pic_buf_desc_init_data.top_padding        = init_data_ptr->top_padding;
         input_pic_buf_desc_init_data.bot_padding        = init_data_ptr->bot_padding;
         input_pic_buf_desc_init_data.color_format       = EB_YUV420; //set to 420 for MD
-        input_pic_buf_desc_init_data.split_mode         = EB_FALSE;
+        input_pic_buf_desc_init_data.split_mode         = FALSE;
         EB_NEW(object_ptr->chroma_downsampled_picture_ptr,
                svt_picture_buffer_desc_ctor,
                (EbPtr)&input_pic_buf_desc_init_data);
-        object_ptr->is_chroma_downsampled_picture_ptr_owner = EB_TRUE;
+        object_ptr->is_chroma_downsampled_picture_ptr_owner = TRUE;
     } else if (init_data_ptr->color_format == EB_YUV420) {
         object_ptr->chroma_downsampled_picture_ptr = NULL;
     } else
@@ -1371,46 +1363,46 @@ EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr,
     // GOP
     object_ptr->pred_struct_index       = 0;
     object_ptr->picture_number          = 0;
-    object_ptr->idr_flag                = EB_FALSE;
+    object_ptr->idr_flag                = FALSE;
     object_ptr->temporal_layer_index    = 0;
     object_ptr->total_num_bits          = 0;
     object_ptr->last_idr_picture        = 0;
     object_ptr->sb_total_count          = picture_sb_width * picture_sb_height;
     object_ptr->sb_total_count_unscaled = object_ptr->sb_total_count;
-    object_ptr->is_pcs_sb_params        = EB_FALSE;
+    object_ptr->is_pcs_sb_params        = FALSE;
 
     object_ptr->data_ll_head_ptr         = (EbLinkedListNode *)NULL;
     object_ptr->app_out_data_ll_head_ptr = (EbLinkedListNode *)NULL;
-    uint8_t block_count;
-    if (init_data_ptr->enable_adaptive_quantization == 1)
-        block_count = 85;
-    else
-        block_count = 1;
-    EB_MALLOC_2D(object_ptr->variance, object_ptr->sb_total_count, block_count);
 
-    uint8_t plane = init_data_ptr->scene_change_detection ? 3 : 0;
-    if (plane) {
+    if (init_data_ptr->calculate_variance) {
+        uint8_t block_count;
+        if (init_data_ptr->enable_adaptive_quantization == 1)
+            block_count = 85;
+        else
+            block_count = 1;
+        EB_MALLOC_2D(object_ptr->variance, object_ptr->sb_total_count, block_count);
+    }
+
+    if (init_data_ptr->scene_change_detection) {
         EB_ALLOC_PTR_ARRAY(object_ptr->picture_histogram, MAX_NUMBER_OF_REGIONS_IN_WIDTH);
 
         for (uint32_t region_in_picture_width_index = 0;
-            region_in_picture_width_index < MAX_NUMBER_OF_REGIONS_IN_WIDTH;
-            region_in_picture_width_index++) { // loop over horizontal regions
+             region_in_picture_width_index < MAX_NUMBER_OF_REGIONS_IN_WIDTH;
+             region_in_picture_width_index++) { // loop over horizontal regions
             EB_ALLOC_PTR_ARRAY(object_ptr->picture_histogram[region_in_picture_width_index],
-                MAX_NUMBER_OF_REGIONS_IN_HEIGHT);
+                               MAX_NUMBER_OF_REGIONS_IN_HEIGHT);
             for (uint32_t region_in_picture_height_index = 0;
-                region_in_picture_height_index < MAX_NUMBER_OF_REGIONS_IN_HEIGHT;
-                region_in_picture_height_index++) {
-                EB_MALLOC_2D(object_ptr->picture_histogram[region_in_picture_width_index]
-                    [region_in_picture_height_index],
-                    plane,
-                    HISTOGRAM_NUMBER_OF_BINS);
+                 region_in_picture_height_index < MAX_NUMBER_OF_REGIONS_IN_HEIGHT;
+                 region_in_picture_height_index++) {
+                EB_MALLOC_ARRAY(object_ptr->picture_histogram[region_in_picture_width_index]
+                                                             [region_in_picture_height_index],
+                                HISTOGRAM_NUMBER_OF_BINS);
             }
         }
     }
 
     if (init_data_ptr->pass == ENC_FIRST_PASS ||
-        (init_data_ptr->rate_control_mode && init_data_ptr->pass == ENC_SINGLE_PASS))
-    {
+        (init_data_ptr->rate_control_mode && init_data_ptr->pass == ENC_SINGLE_PASS)) {
         const uint16_t picture_width_in_mb  = (uint16_t)((init_data_ptr->picture_width + 15) / 16);
         const uint16_t picture_height_in_mb = (uint16_t)((init_data_ptr->picture_height + 15) / 16);
         EB_MALLOC_ARRAY(object_ptr->firstpass_data.mb_stats,
@@ -1431,7 +1423,6 @@ EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr,
     EB_MALLOC_ARRAY(object_ptr->me_8x8_cost_variance, object_ptr->sb_total_count);
     // SB noise variance array
     EB_CREATE_MUTEX(object_ptr->me_processed_b64_mutex);
-    EB_MALLOC_ARRAY(object_ptr->sb_depth_mode_array, object_ptr->sb_total_count);
     EB_CREATE_SEMAPHORE(object_ptr->temp_filt_done_semaphore, 0, 1);
     EB_CREATE_MUTEX(object_ptr->temp_filt_mutex);
     EB_CREATE_MUTEX(object_ptr->debug_mutex);
@@ -1494,17 +1485,23 @@ EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr,
         object_ptr->tile_group_info,
         (object_ptr->av1_cm->tiles_info.tile_rows * object_ptr->av1_cm->tiles_info.tile_cols));
 
-    object_ptr->frame_superres_enabled = EB_FALSE;
+    object_ptr->frame_superres_enabled = FALSE;
     object_ptr->aligned_width          = init_data_ptr->picture_width;
     object_ptr->aligned_height         = init_data_ptr->picture_height;
     object_ptr->frame_width            = init_data_ptr->picture_width;
     object_ptr->frame_height           = init_data_ptr->picture_height;
+    object_ptr->render_width           = init_data_ptr->picture_width;
+    object_ptr->render_height          = init_data_ptr->picture_height;
 
     object_ptr->superres_denom             = SCALE_NUMERATOR;
     object_ptr->superres_total_recode_loop = 0;
     object_ptr->superres_recode_loop       = 0;
     memset(&object_ptr->superres_rdcost, 0, sizeof(object_ptr->superres_rdcost));
     memset(&object_ptr->superres_denom_array, 0, sizeof(object_ptr->superres_denom_array));
+
+    object_ptr->frame_resize_enabled = FALSE;
+    object_ptr->resize_denom         = SCALE_NUMERATOR;
+
     // Loop variables
     object_ptr->loop_count      = 0;
     object_ptr->overshoot_seen  = 0;
@@ -1514,9 +1511,12 @@ EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr,
     EbInputResolution resolution;
     derive_input_resolution(&resolution,
                             init_data_ptr->picture_width * init_data_ptr->picture_height);
-    object_ptr->enable_me_8x8 = !get_disallow_below_16x16_picture_level(
-        init_data_ptr->enc_mode, resolution, B_SLICE, 0, 1, 0);
     object_ptr->enable_me_16x16 = get_enable_me_16x16(init_data_ptr->enc_mode);
+
+    // 8x8 can only be used if 16x16 is enabled
+    object_ptr->enable_me_8x8 = object_ptr->enable_me_16x16
+        ? get_enable_me_8x8(init_data_ptr->enc_mode)
+        : 0;
     return return_error;
 }
 static void me_dctor(EbPtr p) {
@@ -1539,10 +1539,12 @@ static void me_dctor(EbPtr p) {
 EbErrorType me_ctor(MotionEstimationData *object_ptr, EbPtr object_init_data_ptr) {
     PictureControlSetInitData *init_data_ptr    = (PictureControlSetInitData *)object_init_data_ptr;
     EbErrorType                return_error     = EB_ErrorNone;
-    const uint16_t             picture_sb_width = (uint16_t)(
-        (init_data_ptr->picture_width + init_data_ptr->sb_sz - 1) / init_data_ptr->sb_sz);
-    const uint16_t picture_sb_height = (uint16_t)(
-        (init_data_ptr->picture_height + init_data_ptr->sb_sz - 1) / init_data_ptr->sb_sz);
+    const uint16_t             picture_sb_width = (uint16_t)((init_data_ptr->picture_width +
+                                                  init_data_ptr->sb_sz - 1) /
+                                                 init_data_ptr->sb_sz);
+    const uint16_t             picture_sb_height = (uint16_t)((init_data_ptr->picture_height +
+                                                   init_data_ptr->sb_sz - 1) /
+                                                  init_data_ptr->sb_sz);
 
     uint16_t sb_index;
     object_ptr->dctor                   = me_dctor;
@@ -1595,7 +1597,7 @@ EbErrorType me_ctor(MotionEstimationData *object_ptr, EbPtr object_init_data_ptr
         object_ptr->tpl_beta                      = NULL;
         object_ptr->tpl_rdmult_scaling_factors    = NULL;
         object_ptr->tpl_sb_rdmult_scaling_factors = NULL;
-        object_ptr->tpl_src_stats_buffer                 = NULL;
+        object_ptr->tpl_src_stats_buffer          = NULL;
     }
     return return_error;
 }
@@ -1621,21 +1623,23 @@ EbErrorType sb_params_init_pcs(SequenceControlSet *scs_ptr, PictureParentControl
         pcs_ptr->sb_params_array[sb_index].origin_y =
             pcs_ptr->sb_params_array[sb_index].vertical_index * scs_ptr->sb_sz;
 
-        pcs_ptr->sb_params_array[sb_index].width = (uint8_t)(
-            ((encoding_width - pcs_ptr->sb_params_array[sb_index].origin_x) < scs_ptr->sb_sz)
-                ? encoding_width - pcs_ptr->sb_params_array[sb_index].origin_x
-                : scs_ptr->sb_sz);
+        pcs_ptr->sb_params_array[sb_index].width =
+            (uint8_t)(((encoding_width - pcs_ptr->sb_params_array[sb_index].origin_x) <
+                       scs_ptr->sb_sz)
+                          ? encoding_width - pcs_ptr->sb_params_array[sb_index].origin_x
+                          : scs_ptr->sb_sz);
 
-        pcs_ptr->sb_params_array[sb_index].height = (uint8_t)(
-            ((encoding_height - pcs_ptr->sb_params_array[sb_index].origin_y) < scs_ptr->sb_sz)
-                ? encoding_height - pcs_ptr->sb_params_array[sb_index].origin_y
-                : scs_ptr->sb_sz);
+        pcs_ptr->sb_params_array[sb_index].height =
+            (uint8_t)(((encoding_height - pcs_ptr->sb_params_array[sb_index].origin_y) <
+                       scs_ptr->sb_sz)
+                          ? encoding_height - pcs_ptr->sb_params_array[sb_index].origin_y
+                          : scs_ptr->sb_sz);
 
-        pcs_ptr->sb_params_array[sb_index].is_complete_sb = (uint8_t)(
-            ((pcs_ptr->sb_params_array[sb_index].width == scs_ptr->sb_sz) &&
-             (pcs_ptr->sb_params_array[sb_index].height == scs_ptr->sb_sz))
-                ? 1
-                : 0);
+        pcs_ptr->sb_params_array[sb_index].is_complete_sb =
+            (uint8_t)(((pcs_ptr->sb_params_array[sb_index].width == scs_ptr->sb_sz) &&
+                       (pcs_ptr->sb_params_array[sb_index].height == scs_ptr->sb_sz))
+                          ? 1
+                          : 0);
 
         pcs_ptr->sb_params_array[sb_index].is_edge_sb =
             (pcs_ptr->sb_params_array[sb_index].origin_x < scs_ptr->sb_sz) ||
@@ -1657,8 +1661,8 @@ EbErrorType sb_params_init_pcs(SequenceControlSet *scs_ptr, PictureParentControl
                       raster_scan_blk_y[raster_scan_blk_index] +
                       raster_scan_blk_size[raster_scan_blk_index] >
                   encoding_height))
-                ? EB_FALSE
-                : EB_TRUE;
+                ? FALSE
+                : TRUE;
         }
 
         // super-res can not work with multi-tiles, just set up it for no tiling
@@ -1693,21 +1697,23 @@ EbErrorType sb_geom_init_pcs(SequenceControlSet *scs_ptr, PictureParentControlSe
         pcs_ptr->sb_geom[sb_index].origin_y = pcs_ptr->sb_geom[sb_index].vertical_index *
             scs_ptr->sb_size_pix;
 
-        pcs_ptr->sb_geom[sb_index].width = (uint8_t)(
-            ((encoding_width - pcs_ptr->sb_geom[sb_index].origin_x) < scs_ptr->sb_size_pix)
-                ? encoding_width - pcs_ptr->sb_geom[sb_index].origin_x
-                : scs_ptr->sb_size_pix);
+        pcs_ptr->sb_geom[sb_index].width =
+            (uint8_t)(((encoding_width - pcs_ptr->sb_geom[sb_index].origin_x) <
+                       scs_ptr->sb_size_pix)
+                          ? encoding_width - pcs_ptr->sb_geom[sb_index].origin_x
+                          : scs_ptr->sb_size_pix);
 
-        pcs_ptr->sb_geom[sb_index].height = (uint8_t)(
-            ((encoding_height - pcs_ptr->sb_geom[sb_index].origin_y) < scs_ptr->sb_size_pix)
-                ? encoding_height - pcs_ptr->sb_geom[sb_index].origin_y
-                : scs_ptr->sb_size_pix);
+        pcs_ptr->sb_geom[sb_index].height =
+            (uint8_t)(((encoding_height - pcs_ptr->sb_geom[sb_index].origin_y) <
+                       scs_ptr->sb_size_pix)
+                          ? encoding_height - pcs_ptr->sb_geom[sb_index].origin_y
+                          : scs_ptr->sb_size_pix);
 
-        pcs_ptr->sb_geom[sb_index].is_complete_sb = (uint8_t)(
-            ((pcs_ptr->sb_geom[sb_index].width == scs_ptr->sb_size_pix) &&
-             (pcs_ptr->sb_geom[sb_index].height == scs_ptr->sb_size_pix))
-                ? 1
-                : 0);
+        pcs_ptr->sb_geom[sb_index].is_complete_sb =
+            (uint8_t)(((pcs_ptr->sb_geom[sb_index].width == scs_ptr->sb_size_pix) &&
+                       (pcs_ptr->sb_geom[sb_index].height == scs_ptr->sb_size_pix))
+                          ? 1
+                          : 0);
 
         uint16_t max_block_count = scs_ptr->max_block_cnt;
 
@@ -1722,16 +1728,16 @@ EbErrorType sb_geom_init_pcs(SequenceControlSet *scs_ptr, PictureParentControlSe
                      (pcs_ptr->sb_geom[sb_index].origin_y + blk_geom->origin_y +
                           blk_geom->bheight / 2 <
                       encoding_height))
-                    ? EB_TRUE
-                    : EB_FALSE;
+                    ? TRUE
+                    : FALSE;
 
                 if (blk_geom->shape != PART_N)
                     blk_geom = get_blk_geom_mds(blk_geom->sqi_mds);
                 pcs_ptr->sb_geom[sb_index].block_is_inside_md_scan[md_scan_block_index] =
                     ((pcs_ptr->sb_geom[sb_index].origin_x >= encoding_width) ||
                      (pcs_ptr->sb_geom[sb_index].origin_y >= encoding_height))
-                    ? EB_FALSE
-                    : EB_TRUE;
+                    ? FALSE
+                    : TRUE;
             } else {
                 if (blk_geom->shape != PART_N)
                     blk_geom = get_blk_geom_mds(blk_geom->sqi_mds);
@@ -1741,16 +1747,16 @@ EbErrorType sb_geom_init_pcs(SequenceControlSet *scs_ptr, PictureParentControlSe
                       encoding_width) ||
                      (pcs_ptr->sb_geom[sb_index].origin_y + blk_geom->origin_y + blk_geom->bheight >
                       encoding_height))
-                    ? EB_FALSE
-                    : EB_TRUE;
+                    ? FALSE
+                    : TRUE;
 
                 pcs_ptr->sb_geom[sb_index].block_is_inside_md_scan[md_scan_block_index] =
                     ((pcs_ptr->sb_geom[sb_index].origin_x + blk_geom->origin_x + blk_geom->bwidth >
                       encoding_width) ||
                      (pcs_ptr->sb_geom[sb_index].origin_y + blk_geom->origin_y + blk_geom->bheight >
                       encoding_height))
-                    ? EB_FALSE
-                    : EB_TRUE;
+                    ? FALSE
+                    : TRUE;
             }
         }
     }
