@@ -23,7 +23,7 @@
 extern "C" {
 #endif
 typedef struct MiniGopSizeCtrls {
-    int adptive_enable; // 0: Off, 1: Switch at clip level, 2: Switch at GOP level
+    int    adptive_enable; // 0: Off, 1: Switch at clip level, 2: Switch at GOP level
     double short_shot_th; // Threshold to determine short scene.
     double animation_type_th; // Threshold to determine animation scene
     double lm_th; // Threshold to determine low motion scene
@@ -54,6 +54,11 @@ typedef struct MidPassControls {
     uint8_t ds; // use downsampled input
 } MidPassControls;
 
+typedef struct BitstreamLevel {
+    uint8_t major;
+    uint8_t minor;
+} BitstreamLevel;
+
 /************************************
      * Sequence Control Set
      ************************************/
@@ -135,25 +140,25 @@ typedef struct SequenceControlSet {
     uint8_t compound_mode;
 
     /*!< Sequence resolution parameters */
-    uint32_t          chroma_format_idc;
-    uint16_t          subsampling_x; // add chroma subsampling parameters
-    uint16_t          subsampling_y;
-    uint16_t          max_input_luma_width;
-    uint16_t          max_input_luma_height;
-    uint16_t          max_input_chroma_width;
-    uint16_t          max_input_chroma_height;
-    uint16_t          max_input_pad_bottom;
-    uint16_t          max_input_pad_right;
-    uint32_t          chroma_width;
-    uint32_t          chroma_height;
-    uint32_t          pad_right;
-    uint32_t          pad_bottom;
-    uint16_t          left_padding;
-    uint16_t          top_padding;
-    uint16_t          right_padding;
-    uint16_t          bot_padding;
-    uint32_t          frame_rate;
-    uint32_t          encoder_bit_depth;
+    uint32_t chroma_format_idc;
+    uint16_t subsampling_x; // add chroma subsampling parameters
+    uint16_t subsampling_y;
+    uint16_t max_input_luma_width; // input luma width aligned to 8, this is used during encoding
+    uint16_t max_input_luma_height; // input luma height aligned to 8, this is used during encoding
+    uint16_t max_input_chroma_width;
+    uint16_t max_input_chroma_height;
+    uint16_t max_input_pad_bottom;
+    uint16_t max_input_pad_right;
+    uint32_t chroma_width;
+    uint32_t chroma_height;
+    uint32_t pad_right;
+    uint32_t pad_bottom;
+    uint16_t left_padding;
+    uint16_t top_padding;
+    uint16_t right_padding;
+    uint16_t bot_padding;
+    uint32_t frame_rate; //stored in Q16
+    uint32_t encoder_bit_depth;
     EbInputResolution input_resolution;
 
     /*!< Super block parameters set for the stream */
@@ -167,12 +172,6 @@ typedef struct SequenceControlSet {
     uint16_t max_block_cnt;
     /*!< Restoration Unit parameters set for the stream */
     int32_t rest_units_per_tile;
-    /*!< Block limits */
-    uint8_t max_blk_size;
-    uint8_t min_blk_size;
-    uint8_t max_intra_size;
-    uint8_t min_intra_size;
-
     /*!< Sub picture reagions for picture analysis */
     uint32_t picture_analysis_number_of_regions_per_width;
     uint32_t picture_analysis_number_of_regions_per_height;
@@ -236,7 +235,7 @@ typedef struct SequenceControlSet {
     uint32_t         rest_process_init_count;
     uint32_t         tpl_disp_process_init_count;
     uint32_t         total_process_init_count;
-    int32_t          lap_enabled;
+    int32_t          lap_rc;
     TWO_PASS         twopass;
     double           double_frame_rate;
     Quants           quants_bd; // follows input bit depth
@@ -247,6 +246,7 @@ typedef struct SequenceControlSet {
     int32_t          nmv_vec_cost[MV_JOINTS];
     int32_t          nmv_costs[2][MV_VALS];
     uint8_t          mvrate_set;
+    VqCtrls          vq_ctrls;
     MiniGopSizeCtrls mgs_ctls;
     TfControls       tf_params_per_type[3]; // [I_SLICE][BASE][L1]
     MrpCtrls         mrp_ctrls;
@@ -274,7 +274,7 @@ typedef struct SequenceControlSet {
     * 1: 16 bit pipeline.
     * Now 16bit pipeline is only enabled in filter
     * Default is 0. */
-    EbBool is_16bit_pipeline;
+    Bool is_16bit_pipeline;
 
     /* Super block size (mm-signal)
     *
@@ -289,9 +289,7 @@ typedef struct SequenceControlSet {
     /* Global motion
     *
     * Default is 1. */
-    EbBool enable_global_motion;
-    int sg_filter_mode;
-    int wn_filter_mode;
+    Bool enable_global_motion;
 
     /* inter intra compound
     *
@@ -320,11 +318,6 @@ typedef struct SequenceControlSet {
     *
     * Default is -1. */
     int new_nearest_comb_inject;
-
-    /* nsq table
-    *
-    * Default is -1. */
-    int nsq_table;
     /* frame end cdf update
     *
     * Default is -1. */
@@ -344,17 +337,6 @@ typedef struct SequenceControlSet {
     *
     * Default is -1. */
     int compound_level;
-
-    /* Chroma mode
-    *
-    * Level                Settings
-    * CHROMA_MODE_0  0     Full chroma search @ MD
-    * CHROMA_MODE_1  1     Fast chroma search @ MD
-    * CHROMA_MODE_2  2     Chroma blind @ MD + CFL @ EP
-    * CHROMA_MODE_3  3     Chroma blind @ MD + no CFL @ EP
-    *
-    * Default is -1 (AUTO) */
-    int set_chroma_mode;
     /* Disable chroma from luma (CFL)
     *
     * Default is -1 (auto) */
@@ -394,12 +376,6 @@ typedef struct SequenceControlSet {
     *
     * Default is - 1. */
     int pic_based_rate_est;
-
-    /* Flag to enable the use of non-swaure partitions
-    *
-    * Default is 1. */
-    EbBool ext_block_flag;
-
     /* Flag to control intraBC mode
     *  0      OFF
     *  1      slow
@@ -438,31 +414,22 @@ typedef struct SequenceControlSet {
 
     //Flag that will hold the tpl level, set at init time, level 0 is off, other levels are set by preset
     uint8_t tpl_level;
+
+    // If true, calculate and store the SB-based variance
+    uint8_t calculate_variance;
+
+    // Desired dimensions for an externally triggered resize
+    ResizePendingParams resize_pending_params;
 } SequenceControlSet;
-
-typedef struct EbSequenceControlSetInitData {
-    EncodeContext *encode_context_ptr;
-    int32_t        sb_size;
-} EbSequenceControlSetInitData;
-
 typedef struct EbSequenceControlSetInstance {
     EbDctor             dctor;
-    EncodeContext *     encode_context_ptr;
+    EncodeContext      *encode_context_ptr;
     SequenceControlSet *scs_ptr;
-    EbHandle            config_mutex;
 } EbSequenceControlSetInstance;
 
 /**************************************
      * Extern Function Declarations
      **************************************/
-extern EbErrorType svt_sequence_control_set_creator(EbPtr *object_dbl_ptr,
-                                                    EbPtr  object_init_data_ptr);
-
-extern EbErrorType svt_sequence_control_set_ctor(SequenceControlSet *object,
-                                                 EbPtr               object_init_data_ptr);
-
-extern EbErrorType copy_sequence_control_set(SequenceControlSet *dst, SequenceControlSet *src);
-
 extern EbErrorType svt_sequence_control_set_instance_ctor(EbSequenceControlSetInstance *object_ptr);
 
 extern EbErrorType sb_params_init(SequenceControlSet *scs_ptr);
