@@ -193,12 +193,22 @@ TEST_P(PackTest, PackTest) {
 
 INSTANTIATE_TEST_CASE_P(PACK, PackTest, ::testing::ValuesIn(TEST_PACK_SIZES));
 
+// test svt_compressed_packmsb_sse4_1_intrin
 // test svt_compressed_packmsb_avx2_intrin
-class PackMsbTest : public ::testing::TestWithParam<AreaSize> {
+typedef void (*svt_compressed_packmsb_fn)(
+    uint8_t *in8_bit_buffer, uint32_t in8_stride, uint8_t *inn_bit_buffer,
+    uint32_t inn_stride, uint16_t *out16_bit_buffer, uint32_t out_stride,
+    uint32_t width, uint32_t height);
+
+typedef std::tuple<AreaSize, svt_compressed_packmsb_fn>
+    svt_compressed_packmsb_param;
+class PackMsbTest
+    : public ::testing::TestWithParam<svt_compressed_packmsb_param> {
   public:
     PackMsbTest()
-        : area_width_(std::get<0>(GetParam())),
-          area_height_(std::get<1>(GetParam())) {
+        : area_width_(std::get<0>(TEST_GET_PARAM(0))),
+          area_height_(std::get<1>(TEST_GET_PARAM(0))) {
+        tst_fn = TEST_GET_PARAM(1);
         in8_stride_ = out_stride_ = MAX_TEST_SIZE;
         inn_stride_ = in8_stride_ >> 2;
         test_size_ = MAX_TEST_SIZE * MAX_TEST_SIZE;
@@ -252,14 +262,14 @@ class PackMsbTest : public ::testing::TestWithParam<AreaSize> {
         for (int i = 0; i < RANDOM_TIME; i++) {
             svt_buf_random_u8(inn_bit_buffer_, test_size_ >> 2);
             svt_buf_random_u8(in_8bit_buffer_, test_size_);
-            svt_compressed_packmsb_avx2_intrin(in_8bit_buffer_,
-                                               in8_stride_,
-                                               inn_bit_buffer_,
-                                               inn_stride_,
-                                               out_16bit_buffer1_,
-                                               out_stride_,
-                                               area_width_,
-                                               area_height_);
+            tst_fn(in_8bit_buffer_,
+                   in8_stride_,
+                   inn_bit_buffer_,
+                   inn_stride_,
+                   out_16bit_buffer1_,
+                   out_stride_,
+                   area_width_,
+                   area_height_);
 
             svt_compressed_packmsb_c(in_8bit_buffer_,
                                      in8_stride_,
@@ -287,24 +297,42 @@ class PackMsbTest : public ::testing::TestWithParam<AreaSize> {
     uint16_t *out_16bit_buffer1_, *out_16bit_buffer2_;
     uint32_t area_width_, area_height_;
     uint32_t test_size_;
+    svt_compressed_packmsb_fn tst_fn;
 };
 
 TEST_P(PackMsbTest, PackMsbTest) {
     run_test();
 };
 
-INSTANTIATE_TEST_CASE_P(PACKMSB, PackMsbTest,
-                        ::testing::ValuesIn(TEST_PACK_SIZES));
+INSTANTIATE_TEST_CASE_P(
+    PACKMSB, PackMsbTest,
+    ::testing::Combine(::testing::ValuesIn(TEST_PACK_SIZES),
+                       ::testing::Values(svt_compressed_packmsb_sse4_1_intrin,
+                                         svt_compressed_packmsb_avx2_intrin)));
 
-INSTANTIATE_TEST_CASE_P(PACKMSB_EXTEND, PackMsbTest,
-                        ::testing::ValuesIn(TEST_PACK_SIZES_EXTEND));
+INSTANTIATE_TEST_CASE_P(
+    PACKMSB_EXTEND, PackMsbTest,
+    ::testing::Combine(::testing::ValuesIn(TEST_PACK_SIZES_EXTEND),
+                       ::testing::Values(svt_compressed_packmsb_sse4_1_intrin,
+                                         svt_compressed_packmsb_avx2_intrin)));
 
+// test svt_unpack_and_2bcompress_sse4_1
 // test svt_unpack_and_2bcompress_avx2
-class Unpack2bCompress : public ::testing::TestWithParam<AreaSize> {
+typedef void (*svt_unpack_and_2bcompress_fn)(
+    uint16_t *in16b_buffer, uint32_t in16b_stride, uint8_t *out8b_buffer,
+    uint32_t out8b_stride, uint8_t *out2b_buffer, uint32_t out2b_stride,
+    uint32_t width, uint32_t height);
+
+typedef std::tuple<AreaSize, svt_unpack_and_2bcompress_fn>
+    Unpack2bCompressParam;
+
+class Unpack2bCompress
+    : public ::testing::TestWithParam<Unpack2bCompressParam> {
   public:
     Unpack2bCompress()
-        : area_width_(std::get<0>(GetParam())),
-          area_height_(std::get<1>(GetParam())) {
+        : area_width_(std::get<0>(TEST_GET_PARAM(0))),
+          area_height_(std::get<1>(TEST_GET_PARAM(0))) {
+        tst_fn = TEST_GET_PARAM(1);
         out8_stride_ = in_stride_ = MAX_TEST_SIZE;
         out2_stride_ = out8_stride_ >> 2;
         test_size_ = MAX_TEST_SIZE * MAX_TEST_SIZE;
@@ -366,14 +394,14 @@ class Unpack2bCompress : public ::testing::TestWithParam<AreaSize> {
         for (int i = 0; i < RANDOM_TIME; i++) {
             svt_buf_random_u16_with_bd(in_16bit_, test_size_, 10);
 
-            svt_unpack_and_2bcompress_avx2(in_16bit_,
-                                           in_stride_,
-                                           out_8bit_buffer_mod_,
-                                           out8_stride_,
-                                           out_2bit_buffer_mod_,
-                                           out2_stride_,
-                                           area_width_,
-                                           area_height_);
+            tst_fn(in_16bit_,
+                   in_stride_,
+                   out_8bit_buffer_mod_,
+                   out8_stride_,
+                   out_2bit_buffer_mod_,
+                   out2_stride_,
+                   area_width_,
+                   area_height_);
 
             svt_unpack_and_2bcompress_c(in_16bit_,
                                         in_stride_,
@@ -411,30 +439,48 @@ class Unpack2bCompress : public ::testing::TestWithParam<AreaSize> {
     uint16_t *in_16bit_;
     uint32_t area_width_, area_height_;
     uint32_t test_size_;
+    svt_unpack_and_2bcompress_fn tst_fn;
 };
 
 TEST_P(Unpack2bCompress, Unpack2bCompress) {
     run_test();
 };
 
-INSTANTIATE_TEST_CASE_P(UNPACK2BCOMPRESS, Unpack2bCompress,
-                        ::testing::ValuesIn(TEST_PACK_SIZES));
+INSTANTIATE_TEST_CASE_P(
+    UNPACK2BCOMPRESS, Unpack2bCompress,
+    ::testing::Combine(::testing::ValuesIn(TEST_PACK_SIZES),
+                       ::testing::Values(svt_unpack_and_2bcompress_sse4_1,
+                                         svt_unpack_and_2bcompress_avx2)));
 
-INSTANTIATE_TEST_CASE_P(UNPACK2BCOMPRESS_EXTEND, Unpack2bCompress,
-                        ::testing::ValuesIn(TEST_PACK_SIZES_EXTEND));
+INSTANTIATE_TEST_CASE_P(
+    UNPACK2BCOMPRESS_EXTEND, Unpack2bCompress,
+    ::testing::Combine(::testing::ValuesIn(TEST_PACK_SIZES_EXTEND),
+                       ::testing::Values(svt_unpack_and_2bcompress_sse4_1,
+                                         svt_unpack_and_2bcompress_avx2)));
+
+INSTANTIATE_TEST_CASE_P(
+    UNPACK2BCOMPRESS_EXTEND2, Unpack2bCompress,
+    ::testing::Combine(::testing::Values(AreaSize(32, 1), AreaSize(32, 2),
+                                         AreaSize(32, 3), AreaSize(32, 5),
+                                         AreaSize(64, 1), AreaSize(64, 2),
+                                         AreaSize(64, 3), AreaSize(64, 5),
+                                         AreaSize(65, 3), AreaSize(66, 5),
+                                         AreaSize(129, 3), AreaSize(129, 5)),
+                       ::testing::Values(svt_unpack_and_2bcompress_sse4_1,
+                                         svt_unpack_and_2bcompress_avx2)));
 
 // test svt_enc_msb_pack2d_avx2_intrin_al and svt_enc_msb_pack2d_sse2_intrin.
 // There is an implicit assumption that the width should be multiple of 4.
 // Also there are special snippet to handle width of {4, 8, 16, 32, 64}, so use
 // TEST_COMMON_SIZES to cover all the width;
 AreaSize TEST_COMMON_SIZES[] = {
-    AreaSize(4, 4),    AreaSize(4, 8),   AreaSize(8, 4),   AreaSize(8, 8),
-    AreaSize(16, 16),  AreaSize(4, 16),  AreaSize(16, 4),  AreaSize(16, 8),
-    AreaSize(8, 16),   AreaSize(32, 32), AreaSize(32, 8),  AreaSize(16, 32),
-    AreaSize(8, 32),   AreaSize(32, 16), AreaSize(16, 64), AreaSize(64, 16),
-    AreaSize(64, 64),  AreaSize(64, 32), AreaSize(32, 64), AreaSize(128, 128),
-    AreaSize(68, 64),  AreaSize(72, 64), AreaSize(80, 64), AreaSize(96, 64),
-    AreaSize(64, 128), AreaSize(128, 64)};
+    AreaSize(4, 4),    AreaSize(4, 8),    AreaSize(8, 4),   AreaSize(8, 8),
+    AreaSize(16, 16),  AreaSize(4, 16),   AreaSize(16, 4),  AreaSize(16, 8),
+    AreaSize(8, 16),   AreaSize(32, 32),  AreaSize(32, 8),  AreaSize(16, 32),
+    AreaSize(8, 32),   AreaSize(32, 16),  AreaSize(16, 64), AreaSize(64, 16),
+    AreaSize(64, 64),  AreaSize(64, 32),  AreaSize(32, 64), AreaSize(128, 128),
+    AreaSize(68, 64),  AreaSize(72, 64),  AreaSize(80, 64), AreaSize(96, 64),
+    AreaSize(64, 128), AreaSize(128, 64), AreaSize(48, 64), AreaSize(24, 64)};
 
 class Pack2dTest : public ::testing::TestWithParam<AreaSize> {
   public:
@@ -568,8 +614,10 @@ class UnPackTest : public ::testing::TestWithParam<AreaSize> {
         test_size_ = MAX_SB_SQUARE;
         out_8bit_buffer1_ = nullptr;
         out_8bit_buffer2_ = nullptr;
+        out_8bit_buffer3_ = nullptr;
         out_nbit_buffer1_ = nullptr;
         out_nbit_buffer2_ = nullptr;
+        out_nbit_buffer3_ = nullptr;
         in_16bit_buffer_ = nullptr;
     }
 
@@ -578,16 +626,22 @@ class UnPackTest : public ::testing::TestWithParam<AreaSize> {
             reinterpret_cast<uint8_t *>(svt_aom_memalign(32, test_size_));
         out_8bit_buffer2_ =
             reinterpret_cast<uint8_t *>(svt_aom_memalign(32, test_size_));
+        out_8bit_buffer3_ =
+            reinterpret_cast<uint8_t *>(svt_aom_memalign(32, test_size_));
         out_nbit_buffer1_ =
             reinterpret_cast<uint8_t *>(svt_aom_memalign(32, test_size_));
         out_nbit_buffer2_ =
             reinterpret_cast<uint8_t *>(svt_aom_memalign(32, test_size_));
+        out_nbit_buffer3_ =
+            reinterpret_cast<uint8_t *>(svt_aom_memalign(32, test_size_));
         in_16bit_buffer_ = reinterpret_cast<uint16_t *>(
             svt_aom_memalign(32, sizeof(uint16_t) * test_size_));
         memset(out_8bit_buffer1_, 0, test_size_);
         memset(out_8bit_buffer2_, 0, test_size_);
+        memset(out_8bit_buffer3_, 0, test_size_);
         memset(out_nbit_buffer1_, 0, test_size_);
         memset(out_nbit_buffer2_, 0, test_size_);
+        memset(out_nbit_buffer3_, 0, test_size_);
     }
 
     void TearDown() override {
@@ -597,10 +651,14 @@ class UnPackTest : public ::testing::TestWithParam<AreaSize> {
             svt_aom_free(out_8bit_buffer1_);
         if (out_8bit_buffer2_)
             svt_aom_free(out_8bit_buffer2_);
+        if (out_8bit_buffer3_)
+            svt_aom_free(out_8bit_buffer3_);
         if (out_nbit_buffer1_)
             svt_aom_free(out_nbit_buffer1_);
         if (out_nbit_buffer2_)
             svt_aom_free(out_nbit_buffer2_);
+        if (out_nbit_buffer3_)
+            svt_aom_free(out_nbit_buffer3_);
         aom_clear_system_state();
     }
 
@@ -666,6 +724,14 @@ class UnPackTest : public ::testing::TestWithParam<AreaSize> {
                                    out_stride_,
                                    area_width_,
                                    area_height_);
+            svt_enc_msb_un_pack2d_avx2_intrin(in_16bit_buffer_,
+                                              in_stride_,
+                                              out_8bit_buffer3_,
+                                              out_nbit_buffer3_,
+                                              out_stride_,
+                                              out_stride_,
+                                              area_width_,
+                                              area_height_);
 
             check_output(area_width_,
                          area_height_,
@@ -680,13 +746,27 @@ class UnPackTest : public ::testing::TestWithParam<AreaSize> {
                 << "svt_enc_msb_un_pack2d_sse2_intrin failed at " << i
                 << "th test with size (" << area_width_ << "," << area_height_
                 << ")";
+
+            check_output(area_width_,
+                         area_height_,
+                         out_8bit_buffer2_,
+                         out_8bit_buffer3_);
+            check_output(area_width_,
+                         area_height_,
+                         out_nbit_buffer2_,
+                         out_nbit_buffer3_);
+
+            EXPECT_FALSE(HasFailure())
+                << "svt_enc_msb_un_pack2d_avx2_intrin failed at " << i
+                << "th test with size (" << area_width_ << "," << area_height_
+                << ")";
         }
     }
 
     uint16_t *in_16bit_buffer_;
     uint32_t in_stride_, out_stride_;
-    uint8_t *out_8bit_buffer1_, *out_8bit_buffer2_, *out_nbit_buffer1_,
-        *out_nbit_buffer2_;
+    uint8_t *out_8bit_buffer1_, *out_8bit_buffer2_, *out_8bit_buffer3_,
+        *out_nbit_buffer1_, *out_nbit_buffer2_, *out_nbit_buffer3_;
     uint32_t area_width_, area_height_;
     uint32_t test_size_;
 };
